'use server';

import Docker from 'dockerode';
import fs from 'fs';
import path from 'path';


// 创建 Docker 客户端实例
const docker = new Docker();

// 定义执行代码的 Server Action
export const runCode = async (params: { code: string; language: string }) => {
    const { code, language } = params;

    if (!code || !language) {
        throw new Error('Code and language are required.');
    }

    const supportedLanguages = ['c', 'java', 'python'];
    if (!supportedLanguages.includes(language)) {
        throw new Error(`Language '${language}' is not supported. Only 'c', 'java', and 'python' are supported.`);
    }

    // 根据语言选择文件名和编译/运行命令
    const fileName =
        language === 'c' ? 'main.c' :
            language === 'java' ? 'Main.java' :
                'script.py';
    const runCmd =
        language === 'c' ? 'gcc main.c -o main && ./main' :
            language === 'java' ? 'javac Main.java && java Main' :
                'python3 script.py';

    // 创建临时目录用于存储代码文件
    const tempDir = path.join('/tmp', `${Date.now()}`);
    fs.mkdirSync(tempDir, { recursive: true });

    const filePath = path.join(tempDir, fileName);
    fs.writeFileSync(filePath, code);

    try {
        // 创建并启动 Docker 容器
        const container = await docker.createContainer({
            Image: 'fly6516.synology.me:8080/multilang:latest',
            Cmd: ['sh', '-c', runCmd],
            AttachStdout: true,
            AttachStderr: true,
            Tty: false,
            HostConfig: {
                Binds: [`${tempDir}:/workspace`], // 将临时目录挂载到容器
            },
            WorkingDir: '/workspace',
        });

        await container.start();

        const logs = await container.logs({ stdout: true, stderr: true, follow: true });

        let output = '';
        logs.on('data', (data: Buffer) => {
            output += data.toString().replace(/[\x00-\x1F\x7F]/g, ''); // 去除不可见字符
        });

        const exitCode = await container.wait();
        await container.remove();

        if (exitCode.StatusCode === 0) {
            return { output };
        } else {
            // 返回详细的错误信息
            return { error: `Execution failed with exit code ${exitCode.StatusCode}. Output: ${output}` };
        }
    } catch (error) {
        // 捕获并返回详细的错误信息
        return { error: `An error occurred: ${(error as Error).message}` };
    } finally {
        // 清理临时目录
        fs.rmSync(tempDir, { recursive: true, force: true });
    }
};
